#ifndef USERGUID_H_
#define USERGUID_H_

typedef unsigned int   UINT32;
//typedef unsigned   uint32;
//#define PUT_UINT32(addr, data)  (*((volatile uint32 *) (addr)) = (data))

#define GET_UINT32(addr)        (*((volatile UINT32 *) (addr)))


/* Access 32-bit memory-mapped registers */
//#define GET_UINT32(address, pData) (*(pData) = *((volatile UINT32 *)(address)))

#define PUT_UINT32(address, data)  (*((volatile UINT32 *)(address)) = (data))


/* Barrier operations defined using compiler intrinsics */

//#define DataMemoryBarrier()                 asm volatile ("dmb sy ")			/* DMB */
//#define DataSynchronizationBarrier()        asm volatile ("dsb" : : : "memory")	/* DSB */
//#define InstructionSynchronizationBarrier() asm volatile ("isb" : : : "memory")	/* ISB */
#define DataMemoryBarrier()                			/* DMB */
#define DataSynchronizationBarrier()                /* DSB */
#define InstructionSynchronizationBarrier()         /* ISB */


/* Global interrupt enable operations on PRIMASK using compiler intrinsics */

#define DisableInterrupts()                 asm volatile ("cpsid i	\n\t")  /* CPSID I */
#define EnableInterrupts()                  asm volatile ("cpsie i	\n\t")   /* CPSIE I */


/* SCS and NVIC register addresses */

#define CORTEXM1_SYSTICK_CSR          0xE000E010
#define CORTEXM1_SYSTICK_RELOAD       0xE000E014
#define CORTEXM1_SYSTICK_CURRENT      0xE000E018

#define CORTEXM1_NVIC_IRQ_PRI_BASE    0xE000E400
#define CORTEXM1_NVIC_IRQ_SET_ENABLE  0xE000E100
#define CORTEXM1_NVIC_IRQ_CLR_ENABLE  0xE000E180
#define CORTEXM1_NVIC_IRQ_SET_PENDING 0xE000E200
#define CORTEXM1_NVIC_IRQ_CLR_PENDING 0xE000E280

#define CORTEXM1_SCB_ICSR             0xE000ED04
#define CORTEXM1_SCB_SHPR3            0xE000ED20

#define DONT_CARE           1

#define GPIO_BASE_ADDR          0xA0000000

#define TIMER_BASE_ADDR         0xA2000000
#define INTERRUPT_BASE_ADDR     0xA1000000

#define CM1_NVIC_ENABLE_REG     0xE000E100
#define CM1_NVIC_ENABLECLR_REG  0xE000E180

#define CM1_IRQ0_MASK           0x00000001

//#define IRQ_UART_RX_MASK        0x00000001
#define IRQ_TIMER_TICK_MASK     0x00000001

#define INTERRUPT_IRQSOFTINT_OFFSET     0x18
#define INTERRUPT_IRQSOFTINTCLR_OFFSET  0x1C
#define INTERRUPT_IRQENABLE_OFFSET      0x20
#define INTERRUPT_IRQENABLECLR_OFFSET   0x24
#define INTERRUPT_IRQRAWSTATUS_OFFSET   0x28
#define INTERRUPT_IRQSTATUS_OFFSET      0x2C

#define TIMER_LOAD_OFFSET      0x00
#define TIMER_CURRENT_OFFSET   0x04
#define TIMER_CONTROL_OFFSET   0x08
#define TIMER_PRESCALE_OFFSET  0x0C
#define TIMER_INTCLR_OFFSET    0x10

#define TIMER_CONTROL_CONTINUOUS  0x00
#define TIMER_CONTROL_ONESHOT     0x04

#define TIMER_CONTROL_IRQDISABLE  0x00
#define TIMER_CONTROL_IRQENABLE   0x02

#define TIMER_CONTROL_DISABLE     0x00
#define TIMER_CONTROL_ENABLE      0x01

#define TIMER_PRESCALE_DIV2       0x00
// Timer functions
void timer_cfg(unsigned baseAddr, unsigned config, unsigned prescale);
void timer_load(unsigned baseAddr, unsigned loadVal);
void timer_irq_clear(unsigned baseAddr);

// Interrupt functions
void interrupt_irq_enable(unsigned baseAddr, unsigned irqMask);
void interrupt_irq_disable(unsigned baseAddr, unsigned irqMask);
unsigned interrupt_irq_status(unsigned baseAddr);
void cm1_irq_enable(unsigned irqMask);
void cm1_irq_disable(unsigned irqMask);


#define global_intr_enable()	asm volatile ("cpsie if")
#define global_intr_disable()	asm volatile ("cpsid if")

void ConfigureIRQ(UINT32 irq, UINT32 priority);
void ConfigurePendSV(UINT32 priority);
void ConfigTimer(long);

#endif /*USERGUID_H_*/
