/* C Declarations to support example code */
#include "UsersGuid.h"

void interrupt_irq_enable(unsigned baseAddr, unsigned irqMask)
{
    PUT_UINT32(baseAddr+INTERRUPT_IRQENABLE_OFFSET, irqMask);
}

void interrupt_irq_disable(unsigned baseAddr, unsigned irqMask)
{
    PUT_UINT32(baseAddr+INTERRUPT_IRQENABLECLR_OFFSET, irqMask);
}

unsigned interrupt_irq_status(unsigned baseAddr)
{
    unsigned status;
    status = GET_UINT32(baseAddr + INTERRUPT_IRQSTATUS_OFFSET );

    return status;
}

void cm1_irq_enable(unsigned irqMask)
{
    PUT_UINT32(CM1_NVIC_ENABLE_REG, irqMask);
}

void cm1_irq_disable(unsigned irqMask)
{
    PUT_UINT32(CM1_NVIC_ENABLECLR_REG, irqMask);
}

void timer_cfg(unsigned baseAddr, unsigned config, unsigned prescale)
{
    PUT_UINT32(baseAddr+TIMER_PRESCALE_OFFSET, prescale);
    PUT_UINT32(baseAddr+TIMER_CONTROL_OFFSET, config);
}

void timer_load(unsigned baseAddr, unsigned loadVal)
{
    PUT_UINT32(baseAddr+TIMER_LOAD_OFFSET, loadVal);
}

void timer_irq_clear(unsigned baseAddr)
{
    PUT_UINT32(baseAddr+TIMER_INTCLR_OFFSET, DONT_CARE);
}

void gpio_out(unsigned baseAddr, unsigned dataOut)
{
    PUT_UINT32(baseAddr , dataOut);
}

void ConfigureIRQ(UINT32 irq, UINT32 priority){
	/* Must be called with the interrupt disabled */
	UINT32 ipri;

	/* Calculate register offset for Interrupt Priority Register */
	UINT32 reg_offset = 4 * (irq / 4);

	/* Calculate bit offset within Interrupt Priority Register */
	UINT32 bit_offset = 8 * (irq % 4) + 6;

	/* All accesses to NVIC registers must be 32-bits so read */
	/* the Interrupt Priority Register to preserve other fields */
	ipri = GET_UINT32(CORTEXM1_NVIC_IRQ_PRI_BASE + reg_offset);

	/* Mask out previous priority */
	ipri &= ~(0x3 << bit_offset);

	/* Set new priority */
	ipri |= (priority << bit_offset);

	/* Update Interrupt Priority Register with new priority */
	PUT_UINT32(CORTEXM1_NVIC_IRQ_PRI_BASE + reg_offset, ipri);

	/* Ensure any previous interrupt is un-pended to avoid */
	/* a spurious interrupt exception entry when it is enabled */
	PUT_UINT32(CORTEXM1_NVIC_IRQ_CLR_PENDING,  1<<irq);

	/* Set bit in Interrupt Set Enable Register to enable IRQ */
	PUT_UINT32(CORTEXM1_NVIC_IRQ_SET_ENABLE,  1<<irq);

	/* Ensure all stores complete */
	DataSynchronizationBarrier();

	return;
}

/* Configure priority of PendSV exception */
void ConfigurePendSV(UINT32 priority)
{
	UINT32 syspri;

	/* The PendSV exception is always enabled so set PRIMASK */
	/* to prevent it from occurring while being configured */
	DisableInterrupts();

	/* All accesses to SCS registers must be 32-bits so read */
	/* the System Handler Priority Register 3 to preserve other fields */
	syspri = GET_UINT32(CORTEXM1_SCB_SHPR3);

	/* Mask out previous priority */
	syspri &= ~(0x3 << 22);

	/* Set new priority */
	syspri |= (priority << 22);

	/* Update System Handler Priority Register 3 with new priority */
	PUT_UINT32(CORTEXM1_SCB_SHPR3, syspri);

	/* Ensure the effect of the priority change occurs before */
	/* clearing PRIMASK to ensure that future PendSV exceptions */
	/* are taken at the new priority */
	DataSynchronizationBarrier();
	InstructionSynchronizationBarrier();

	EnableInterrupts();

	return;
}

void ConfigTimer(long time)
{
    // Configure Timer: Continuous mode, IRQ Enable, Timer Enable, Prescale as defined above
    timer_load(TIMER_BASE_ADDR, time );
    timer_cfg(TIMER_BASE_ADDR, 
    	TIMER_CONTROL_CONTINUOUS | TIMER_CONTROL_IRQENABLE 
    	| TIMER_CONTROL_ENABLE, TIMER_PRESCALE_DIV2 );

    // Enable interrupts
    interrupt_irq_enable(INTERRUPT_BASE_ADDR, 
    	 IRQ_TIMER_TICK_MASK);
    cm1_irq_enable(CM1_IRQ0_MASK);
}

